/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced;

import java.util.concurrent.Executor;

import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.thread.IReusableThread;
import org.ow2.util.pool.impl.enhanced.impl.thread.ThreadPool;
import org.ow2.util.pool.impl.enhanced.impl.util.ExecutorProvider;
import org.ow2.util.pool.impl.enhanced.internal.resizer.impl.shared.SharedResizerPoolThreadManager;
import org.ow2.util.pool.impl.enhanced.manager.IPoolManager;

/**
 * @author Gael Lalire
 */
public class EnhancedPoolFactory {

    /**
     * Pool of reusable thread.
     */
    private IPool<IReusableThread> pool;

    public EnhancedPoolFactory(final IPool<IReusableThread> pool) {
        this.pool = pool;
    }

    /**
     * You should use this method to create {@link EnhancedPool}.
     * @param <E> pool item type
     * @param poolManager a pool manager
     * @return a new pool
     */
    public <E> EnhancedPool<E> createEnhancedPool(final IPoolManager<E> poolManager) {
        return createEnhancedPool(poolManager, false, true);
    }

    /**
     * @param <E> pool item type
     * @param poolManager a pool manager
     * @param setSizeShared if true setExpectiveSize will be called in a
     *        separate thread, you should prefers the schedulerShared option
     *        because on a remove call the creation will be on self thread.
     * @param schedulerShared each time a pool item is needed a shared executor
     *        will be used
     * @return a new pool
     */
    public <E> EnhancedPool<E> createEnhancedPool(final IPoolManager<E> poolManager, final boolean setSizeShared,
            final boolean schedulerShared) {
        Executor executor;
        if (schedulerShared) {
            executor = new ThreadPool(pool, Long.valueOf(0L));
        } else {
            executor = ExecutorProvider.SELF_THREAD_EXECUTOR;
        }
        if (setSizeShared) {
            return new EnhancedPool<E>(poolManager, ResizerType.SHARED_ASYNCHRONOUS, new SharedResizerPoolThreadManager(pool),
                    executor);
        } else {
            return new EnhancedPool<E>(poolManager, ResizerType.SIMPLE, null, executor);
        }
    }

}
