/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.api;

/**
 * A pool is a class which keep some pool item.
 * @author Gael Lalire
 * @param <E> pool item type
 */
public interface IPool<E> {

    /**
     * This method block is there is no available pool item
     * in the pool.
     * @param timeout if the get wait more than timeout then the method
     * throw a {@link TimeoutPoolException}.
     * @return a pool item
     * @throws PoolException if error occurs
     */
    E get(IWaitControl timeout) throws PoolException, InterruptedException;

    /**
     * This method must be called when you finished to use
     * the pool item.
     * @param poolItem a pool item
     * @throws NotABusyPoolItemException if error occurs
     */
    void put(final E poolItem) throws NotABusyPoolItemException;

    /**
     * This method can be called when you finished to use
     * the pool item instead of put method.
     *
     * If you call this method the pool item will be replace
     * with a new.
     *
     * @param poolItem a pool item
     * @throws NotABusyPoolItemException if error occurs
     */
    void remove(final E poolItem) throws NotABusyPoolItemException;

    /**
     * Ask pool to recall all timeout.waitOnXXX. So if you change a
     * {@link org.ow2.util.pool.impl.enhanced.impl.waitcontrol.WaitAuthorization}
     * state you can avoid all waiters.
     */
    void signalAllWaiters();

}
