/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.api;

import java.util.concurrent.locks.Condition;

import org.ow2.util.pool.impl.enhanced.impl.waitcontrol.WaitAuthorization;
import org.ow2.util.pool.impl.enhanced.internal.conditionWaitPart.IConditionWaitPart;

/**
 * @author Gael Lalire
 */
public interface IWaitControl {

    void pushWaitAuthorization(final WaitAuthorization waitAuthorization);

    void popWaitAuthorization();

    /**
     * @param condition a condition
     * @return false if timeout, true if notify
     * @throws InterruptedException
     */
    boolean waitOnConditionWaitPart(IConditionWaitPart condition) throws InterruptedException, WaiterInterruptedException;

    /**
     * @param condition a condition
     * @return false if timeout, true if notify
     * @throws InterruptedException
     */
    boolean waitOnCondition(Condition condition) throws InterruptedException, WaiterInterruptedException;

    /**
     * @param condition a condition
     * @return false if timeout, true if notify
     * @throws InterruptedException
     */
    boolean waitOnMutex(Object mutex) throws InterruptedException, WaiterInterruptedException;

    /**
     * This method throw an exception only if on waitOnXXX throw an exception before.
     * If popWaitAuthorization was called the exception may not be thrown again.
     * @throws WaiterInterruptedException
     */
    void verifyInterrupted() throws WaiterInterruptedException;

    /**
     * @return false if timeout
     */
    boolean canContinueToWait();

}
