/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.api.basic;

import org.ow2.util.pool.api.IPoolState;
import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.IllegalTimeoutException;
import org.ow2.util.pool.impl.enhanced.api.TimeoutPoolException;
import org.ow2.util.pool.impl.enhanced.api.WaiterInterruptedException;
import org.ow2.util.pool.impl.enhanced.api.resizer.IResizablePool;

/**
 * @author Gael Lalire
 * @param <E> pool item type
 */
public interface IBasicPool<E> extends IResizablePool<E> {

    /**
     * This method block is there is no available pool item in the pool. It is
     * same as get(INFINITE_TIMEOUT);
     * @return a pool item
     * @throws WaiterInterruptedException if error occurs
     */
    E get() throws WaiterInterruptedException, PoolFactoryBroken, InterruptedException;

    E get(IWaitControl timeout) throws TimeoutPoolException, IllegalTimeoutException, WaiterInterruptedException,
            PoolFactoryBroken, InterruptedException;

    /**
     * After calling this method the pool will have a zero size and no item will
     * be created until setExpectedSize will be called again.
     */
    void clearPool() throws InterruptedException;

    /**
     * After calling this method all pending remove calls will be called.
     */
    void waitAllRemoveListenerCompleted() throws InterruptedException;

    void lockSizeToZero();

    void unlockSizeToZero();

    /**
     * Fill state on the given object.
     * @param state the state object to populate
     */
    void fillState(final IPoolState state);

}
