/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.api.basic.accessmanager;

import java.util.List;

/**
 *
 * @author Gael Lalire
 * @param <E> pool item type
 */
public interface IAccessManager<E> {

    /**
     * If you return this value it mean you do not want to choose an
     * item.
     */
    int DELAY_GET = -1;

    /**
     * @param poolItemList the list of pool item
     * @return a value between 0 and poolItemList.size()-1
     */
    int choosePoolItemToRelease(List<? extends E> poolItemList);

    /**
     * @param poolItemList the list of pool item
     * @return a value between 0 and poolItemList.size()-1 or
     * DELAY_GET.
     */
    int choosePoolItemToGet(List<? extends E> poolItemList);

    /**
     * @param poolItemList the list of pool item
     * @param poolItem poolItem to put in list
     * @return a value between 0 and poolItemList.size()
     */
    int putPoolItem(List<? extends E> poolItemList, E poolItem);

    /**
     * @param poolItemList the list of pool item
     * @param poolItem poolItem to put in list
     * @return a value between 0 and poolItemList.size()
     */
    int createPoolItem(List<? extends E> poolItemList, E poolItem);

    /**
     * This callback is useful to clean up data you keep by poolItem.
     * @param poolItem poolItem
     */
    void removePoolItem(E poolItem);

}
