/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.api.recorder;


/**
 * Methods of this interface have to be thread safe.
 * @param <E> pool item type
 * @param <I> identifier type
 * @author Gael Lalire
 */
public interface IPoolItemRecorder<E, I> {

    /**
     * This method must not modify poolItem while recording, so if the record is
     * aborted the poolItem can be returned without an useless fetch.
     *
     * The latestIdentifier will not be use in future so if you are not reusing it (return
     * the same identifier) you should remove data keeped by latestIdentifier.
     *
     * @param poolItem a pool item to record
     * @param latestIdentifier the latest identifier fetch (below) method returns
     * @return an identifier which will be used to fetch this item
     */
    I record(E poolItem, I latestIdentifier);

    /**
     * The identifier has to still be valid after the fetch, so if fetch is aborted we
     * can call again the method in future.
     *
     * @param identifier this information contains information to fetch the item
     * @return a fetched item
     */
    E fetch(I identifier);

    /**
     * The identifier will not be use in future because the item has been removed from pool or
     * the pool is finalized for example.
     * @param identifier an identifier
     */
    void remove(I identifier);

}
