/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.api.recorder;

/**
 * @author Gael Lalire
 */
public enum RecorderPoolItemState {

    /**
     * Fetched state.
     */
    FETCHED(false, true),

    /**
     * Recording state.
     */
    RECORDING(true, false),

    /**
     * Recorded state.
     */
    RECORDED(true, false),

    /**
     * Fetching state.
     */
    FETCHING(false, true),

    /**
     * Aborted recording state.
     */
    ABORTED_RECORDING(false, true),

    /**
     * Aborted fetching state.
     */
    ABORTED_FETCHING(true, false);

    /**
     * True if state can be fetched.
     */
    private boolean fetchable;

    /**
     * True if state can be recorded.
     */
    private boolean recordable;

    /**
     * @param fetchable true if state can be fetched
     * @param recordable true if state can be recorded
     */
    private RecorderPoolItemState(final boolean fetchable, final boolean recordable) {
        this.fetchable = fetchable;
        this.recordable = recordable;
    }

    /**
     * @return true if state can be fetched
     */
    public boolean isFetchable() {
        return fetchable;
    }

    /**
     * @return true if state can be recorded
     */
    public boolean isRecordable() {
        return recordable;
    }

}
