/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.api.recorder.accesmanager;

import java.util.List;

import org.ow2.util.pool.impl.enhanced.api.basic.accessmanager.IAccessManager;
import org.ow2.util.pool.impl.enhanced.api.recorder.IRecorderPoolItem;
import org.ow2.util.pool.impl.enhanced.api.recorder.RecorderPoolItemState;

/**
 * @param <E> pool item type
 * @param <I> identifier type
 * @author Gael Lalire
 */
public interface IRecorderAccessManager<E, I> extends IAccessManager<IRecorderPoolItem<? extends E, ? extends I>> {

    /**
     * Ask to choose a RECORDING or ABORTED_FETCHING or RECORDED pool item.
     * If you choose an item which state is not listed above then the pool have to make a correct
     * choice ignoring your response.
     * @param poolItemList the list of pool item
     * @return a value between 0 and poolItemList.size()-1
     * If you return an
     */
    int choosePoolItemToRecord(List<? extends IRecorderPoolItem<? extends E, ? extends I>> poolItemList);

    /**
     * @param poolItemList the list of pool item
     * @return a value between 0 and poolItemList.size()-1 or
     * NO_POOL_ITEM_TO_FETCH if you do not want to fetch any item
     */
    int choosePoolItemToFetch(List<? extends IRecorderPoolItem<? extends E, ? extends I>> poolItemList);

    /**
     * @param poolItemList the list of pool item
     * @param poolItem the pool item
     * @param previousState the previous state of item (to get new state use poolItem.getState())
     * @return a distance (for example 0 : do not move, 1 : move forward, -5 : move 5 time backwards)
     */
    int movePoolItemWhoseStateChanged(List<? extends IRecorderPoolItem<? extends E, ? extends I>> poolItemList,
            IRecorderPoolItem<? extends E, ? extends I> poolItem, RecorderPoolItemState previousState);

}
