/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.callback;

import org.ow2.util.pool.impl.enhanced.api.basic.CreatePoolItemException;
import org.ow2.util.pool.impl.enhanced.api.basic.IPoolItemFactory;
import org.ow2.util.pool.impl.enhanced.impl.PoolError;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.IArgumentsCallable;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.StoppableExecution;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.StoppedExecutionException;

/**
 * @author Gael Lalire
 * @param <E>
 */
public class CreateCB<E> extends StoppableExecution<E> implements IPoolItemFactory<E> {

    public CreateCB(final IPoolItemFactory<E> create) {
        super(new IArgumentsCallable<E>() {

            public E call(final Object... args) throws Exception {
                return create.createPoolItem();
            }

        });
    }

    public E createPoolItem() throws CreatePoolItemException {
        try {
            return call();
        } catch (CreatePoolItemException e) {
            throw e;
        } catch (StoppedExecutionException e) {
            throw new CreatePoolItemException(null, e);
        } catch (RuntimeException e) {
            throw new CreatePoolItemException(null, e);
        } catch (Exception e) {
            throw new PoolError("createPoolItem could only throw CreatePoolItemException as checked exception", e);
        }
    }

}
