/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.callback;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.impl.enhanced.api.basic.IPoolItemRemoveListener;
import org.ow2.util.pool.impl.enhanced.impl.PoolError;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.IArgumentsCallable;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.StoppableExecution;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.StoppedExecutionException;

/**
 * @author Gael Lalire
 * @param <E>
 */
public class RemoveCB<E> extends StoppableExecution<Void> implements IPoolItemRemoveListener<E> {

    private static final Log LOG = LogFactory.getLog(RemoveCB.class);

    public RemoveCB(final IPoolItemRemoveListener<E> removeListener) {
        super(new IArgumentsCallable<Void>() {

            @SuppressWarnings("unchecked")
            public Void call(final Object... args) throws Exception {
                removeListener.poolItemRemoved((E) args[0]);
                return null;
            }

        });
    }

    public void poolItemRemoved(final E poolItem) {
        try {
            call(poolItem);
        } catch (StoppedExecutionException e) {
            LOG.warn("poolItemRemoved failed", e);
        } catch (RuntimeException e) {
            LOG.warn("poolItemRemoved failed", e);
        } catch (Exception e) {
            throw new PoolError("poolItemRemoved could not throw checked exception", e);
        }
    }

}
