/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.deprecated;

import org.ow2.util.pool.api.PoolException;


/**
 * Allows to call specific methods for each implementation of the pool for each lifecycle on an object.
 * @param <InstanceType> the type of the object that are managed by the pool
 *        (could be EasyBeansSLSB, etc.)
 * @param <Clue> a clue which is given to match an instance in the pool
 * @author Florent Benoit
 * @deprecated
 */
@Deprecated
public interface PoolFactory<InstanceType, Clue> {


    /**
     * Creates an instance with the given hint.
     * @param clue always null with this implementation.
     * @throws PoolException if instance cannot be created.
     * @return the created instance.
     */
    InstanceType create(Clue clue) throws PoolException;

    /**
     * Checks if the given object with the given clue is matching.
     * @param object given object against which the check should be done.
     * @param clue the object used as clue to check the matching.
     * @return true if it is matching, else false.
     */
    boolean isMatching(InstanceType object, Clue clue);

    /**
     * Change item clue.
     * @param object given object against which the check should be done.
     * @param clue the object used as clue to check the matching.
     */
    void setClue(InstanceType object, Clue clue);


    /**
     * Callback called when object is gonna be removed.
     * @param object that is being removed from the pool.
     */
    void remove(InstanceType object);

    /**
     * This method is called just before an instance is getted.
     * You should reinit fields you set to null and saved to
     * file system for example.
     * @param object the pool item which will become busy
     * @param clue the clue used to get this instance
     */
    void activate(InstanceType object, Clue clue);

    /**
     * This method is called while an instance is put in pool.
     * You may back up and set to null some field.
     * @param object the pool item which will become available
     */
    void passivate(InstanceType object);

    /**
     * Validate an instance by giving some statistics.
     * @param instance the instance to validate
     * @return true if the element is valid, else false.
     */
    boolean validate(InstanceType instance);

}
