/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.basic.accessmanager.clue;

import java.util.List;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.impl.enhanced.api.basic.clue.accessmanager.IClueAccessManager;
import org.ow2.util.pool.impl.enhanced.api.clue.IClueManager;
import org.ow2.util.pool.impl.enhanced.impl.basic.accessmanager.LastUsedAccessManager;


/**
 *
 * @author Gael Lalire
 * @param <E> pool item type
 * @param <C> clue type
 */
public class LastUsedClueAccessManager<E, C> extends LastUsedAccessManager<E> implements IClueAccessManager<E, C> {

    private static final Log LOG = LogFactory.getLog(LastUsedClueAccessManager.class);

    private IClueManager<E, C> clueManager;

    public LastUsedClueAccessManager(final IClueManager<E, C> clueManager) {
        this.clueManager = clueManager;
    }

    public int choosePoolItemToGet(final List<? extends E> poolItemList, final C clue) {
        int lastIndex = poolItemList.size() - 1;
        int index = lastIndex;
        E poolItem;
        while (index >= 0) {
            poolItem = poolItemList.get(index);
            if (clueManager.tryMatch(poolItem, clue)) {
                LOG.debug("We choose matching clue {0}.", poolItem);
                return index;
            }
            index--;
        }
        LOG.debug("Delay get");
        return DELAY_GET;
    }

    @Override
    public void removePoolItem(final E poolItem) {
        super.removePoolItem(poolItem);
        clueManager.unMatch(poolItem);
    }

    @Override
    public int putPoolItem(final java.util.List<? extends E> poolItemList, final E poolItem) {
        int pos = super.putPoolItem(poolItemList, poolItem);
        clueManager.unMatch(poolItem);
        return pos;
    }

}
