/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.clue.basiccluemanager;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.pool.impl.enhanced.api.clue.IClueManager;
import org.ow2.util.pool.impl.enhanced.api.clue.basiccluemanager.IClueAccessor;

/**
 * @author Gael Lalire
 * @param <E> pool item type
 * @param <C> clue type
 */
public class BasicClueManager<E, C> implements IClueManager<E, C> {

    private List<C> busyClue;

    private boolean clueResetable;

    private IClueAccessor<E, C> clueAccessor;

    /**
     * Less restriction constructor.
     * Allow both reset and share of clue.
     */
    public BasicClueManager(final IClueAccessor<E, C> clueAccessor) {
        this(clueAccessor, true, true);
    }

    public BasicClueManager(final IClueAccessor<E, C> clueAccessor, final boolean clueShareable, final boolean clueResetable) {
        this.clueAccessor = clueAccessor;
        this.clueResetable = clueResetable;
        if (!clueShareable) {
            busyClue = new ArrayList<C>();
        }
    }

    public boolean tryMatch(final E item, final C clue) {
        if (clue == null) {
            if (clueAccessor.getClue(item) == null) {
                return true;
            }
            if (clueResetable) {
                clueAccessor.setClue(item, null);
                return true;
            }
            return false;
        }
        if (busyClue != null && busyClue.contains(clue)) {
            return false;
        }
        C itemClue = clueAccessor.getClue(item);
        if (itemClue == null) {
            clueAccessor.setClue(item, clue);
            if (busyClue != null) {
                busyClue.add(clue);
            }
            return true;
        }
        if (itemClue.equals(clue)) {
            if (busyClue != null) {
                busyClue.add(clue);
            }
            return true;
        }
        if (clueResetable) {
            clueAccessor.setClue(item, clue);
            if (busyClue != null) {
                busyClue.add(clue);
            }
            return true;
        }
        return false;
    }

    public void unMatch(final E item) {
        if (busyClue != null) {
            busyClue.remove(clueAccessor.getClue(item));
        }
    }

}
