/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.listener.clue;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.PoolException;
import org.ow2.util.pool.impl.enhanced.api.clue.ICluePool;
import org.ow2.util.pool.impl.enhanced.api.listener.clue.ICluePoolListener;
import org.ow2.util.pool.impl.enhanced.impl.listener.PoolListenerDispatcherPool;

/**
 *
 * @author Gael Lalire
 * @param <E> pool item type
 * @param <C> clue type
 */
public class CluePoolListenerDispatcherPool<E, C> extends PoolListenerDispatcherPool<E> implements ICluePool<E, C> {

    private ICluePool<E, C> cluePool;

    private List<ICluePoolListener<E, C>> cluePoolListenerList;

    public CluePoolListenerDispatcherPool(final ICluePool<E, C> cluePool) {
        super(cluePool);
        this.cluePool = cluePool;
        cluePoolListenerList = new ArrayList<ICluePoolListener<E, C>>();
    }

    public E get(final C clue, final IWaitControl timeout) throws PoolException, InterruptedException {
        for (ICluePoolListener<E, C> poolListener : cluePoolListenerList) {
            poolListener.getMethodCalled(clue, timeout);
        }
        try {
            E result = cluePool.get(clue, timeout);
            for (ICluePoolListener<E, C> poolListener : cluePoolListenerList) {
                poolListener.getMethodReturned(result, clue, timeout);
            }
            return result;
        } catch (PoolException poolException) {
            for (ICluePoolListener<E, C> poolListener : cluePoolListenerList) {
                poolListener.getMethodFailed(poolException, clue, timeout);
            }
            throw poolException;
        }
    }

    public void addCluePoolListener(final ICluePoolListener<E, C> poolListener) {
        cluePoolListenerList.add(poolListener);
        addPoolListener(poolListener);
    }

    public void removeCluePoolListener(final ICluePoolListener<E, C> poolListener) {
        cluePoolListenerList.remove(poolListener);
        removePoolListener(poolListener);
    }


}
