/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.listener.resizer;

import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.listener.statistics.IPoolStatsListener;
import org.ow2.util.pool.impl.enhanced.impl.listener.PoolListenerDispatcherPool;
import org.ow2.util.pool.impl.enhanced.internal.resizer.api.IResizer;

/**
 *
 * @author Gael Lalire
 * @param <E> pool item type
 * @param <S>
 */
public class StatsResizerPool<E, S> extends PoolListenerDispatcherPool<E> {

    private ResizerUpdatePoolListener<E, S> resizerUpdatePoolListener;

    /**
     * @param pool pool to get stats
     * @param poolStatsListener fill and access stats
     * @param instance an stat instance
     * @param resizer the resizer call resize method (on another pool for example)
     */
    public StatsResizerPool(final IPool<E> pool, final IPoolStatsListener<E, S> poolStatsListener, final S instance,
            final IResizer<S> resizer) {
        super(pool);
        addPoolListener(poolStatsListener);
        resizerUpdatePoolListener = new ResizerUpdatePoolListener<E, S>(instance, resizer, poolStatsListener);
        addPoolListener(resizerUpdatePoolListener);
    }

    public void prepareUpdate() {
        resizerUpdatePoolListener.prepareUpdate();
    }

    public void update() {
        resizerUpdatePoolListener.update();
    }

}
