/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.listener.resizer.clue;

import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.PoolException;
import org.ow2.util.pool.impl.enhanced.api.listener.clue.ICluePoolListener;
import org.ow2.util.pool.impl.enhanced.api.listener.statistics.clue.ICluePoolStatsListener;
import org.ow2.util.pool.impl.enhanced.impl.listener.resizer.ResizerUpdatePoolListener;
import org.ow2.util.pool.impl.enhanced.impl.listener.resizer.UpdateMethod;
import org.ow2.util.pool.impl.enhanced.internal.resizer.api.IResizer;

/**
 *
 * @author Gael Lalire
 * @param <E> pool item type
 * @param <C> clue type
 * @param <S>
 */
public class ResizerUpdateCluePoolListener<E, C, S> extends ResizerUpdatePoolListener<E, S> implements ICluePoolListener<E, C> {

    private UpdateMethod getClueUpdateMethod;

    public ResizerUpdateCluePoolListener(final S stats, final IResizer<S> poolResizer,
            final ICluePoolStatsListener<E, C, S> poolStatsAccessor) {
        this(stats, poolResizer, poolStatsAccessor, UpdateMethod.BEFORE, UpdateMethod.BEFORE, UpdateMethod.AFTER,
                UpdateMethod.AFTER, UpdateMethod.AFTER);
    }

    public ResizerUpdateCluePoolListener(final S stats, final IResizer<S> poolResizer,
            final ICluePoolStatsListener<E, C, S> poolStatsAccessor, final UpdateMethod getClueUpdateMethod,
            final UpdateMethod getUpdateMethod, final UpdateMethod putUpdateMethod, final UpdateMethod removeUpdateMethod,
            final UpdateMethod interruptAllWaitersUpdateMethod) {
        super(stats, poolResizer, poolStatsAccessor, getUpdateMethod, putUpdateMethod, removeUpdateMethod,
                interruptAllWaitersUpdateMethod);
        this.getClueUpdateMethod = getClueUpdateMethod;
    }

    public void getMethodCalled(final C clue, final IWaitControl timeout) {
        prepareUpdate();
        if (getClueUpdateMethod.isBefore()) {
            update();
        }
    }

    public void getMethodFailed(final PoolException poolException, final C clue, final IWaitControl timeout) {
        prepareUpdate();
        if (getClueUpdateMethod.isAfterFailed()) {
            update();
        }
    }

    public void getMethodReturned(final E result, final C clue, final IWaitControl timeout) {
        prepareUpdate();
        if (getClueUpdateMethod.isAfterSuccessful()) {
            update();
        }
    }

}
