/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.listener.resizer.clue;

import org.ow2.util.pool.impl.enhanced.api.clue.ICluePool;
import org.ow2.util.pool.impl.enhanced.api.listener.statistics.clue.ICluePoolStatsListener;
import org.ow2.util.pool.impl.enhanced.impl.listener.clue.CluePoolListenerDispatcherPool;
import org.ow2.util.pool.impl.enhanced.internal.resizer.api.IResizer;

/**
 *
 * @author Gael Lalire
 * @param <E> pool item type
 * @param <C> clue type
 * @param <S>
 */
public class StatsResizerCluePool<E, C, S> extends CluePoolListenerDispatcherPool<E, C> {

    private ResizerUpdateCluePoolListener<E, C, S> resizerUpdateCluePoolListener;

    /**
     * @param pool pool to get stats
     * @param poolStatsListener fill and access stats
     * @param instance an stat instance
     * @param resizer the resizer call resize method (on another pool for example)
     */
    public StatsResizerCluePool(final ICluePool<E, C> pool, final ICluePoolStatsListener<E, C, S> poolStatsListener,
            final S instance, final IResizer<S> resizer) {
        super(pool);
        addCluePoolListener(poolStatsListener);
        resizerUpdateCluePoolListener = new ResizerUpdateCluePoolListener<E, C, S>(instance, resizer, poolStatsListener);
        addCluePoolListener(resizerUpdateCluePoolListener);
    }

    public void prepareUpdate() {
        resizerUpdateCluePoolListener.prepareUpdate();
    }

    public void update() {
        resizerUpdateCluePoolListener.update();
    }

}
