/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.listener.statistics;

import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.listener.statistics.IPoolStatsListener;
import org.ow2.util.pool.impl.enhanced.impl.listener.PoolListenerAdapter;

/**
 * @author Gael Lalire
 * @param <E> pool item type
 */
public class BasicPoolStatsFillerPoolListener<E> extends PoolListenerAdapter<E> implements
        IPoolStatsListener<E, BasicPoolStats> {

    private BasicPoolStats basicPoolStats;

    public BasicPoolStatsFillerPoolListener() {
        this.basicPoolStats = new BasicPoolStats();
    }

    /**
     * @return the basicPoolStats
     */
    protected BasicPoolStats getBasicPoolStats() {
        return basicPoolStats;
    }

    @Override
    public void getMethodCalled(final IWaitControl timeout) {
        synchronized (basicPoolStats) {
            basicPoolStats.setWaiters(basicPoolStats.getWaiters() + 1);
        }
    }

    @Override
    public void getMethodFailed(final Exception poolException, final IWaitControl timeout) {
        synchronized (basicPoolStats) {
            basicPoolStats.setWaiters(basicPoolStats.getWaiters() - 1);
        }
    }

    @Override
    public void getMethodReturned(final E result, final IWaitControl timeout) {
        synchronized (basicPoolStats) {
            basicPoolStats.setNotSendedBack(basicPoolStats.getNotSendedBack() + 1);
            basicPoolStats.setWaiters(basicPoolStats.getWaiters() - 1);
        }
    }

    @Override
    public void putMethodReturned(final E poolItem) {
        synchronized (basicPoolStats) {
            basicPoolStats.setNotSendedBack(basicPoolStats.getNotSendedBack() - 1);
        }
    }

    @Override
    public void removeMethodReturned(final E poolItem) {
        synchronized (basicPoolStats) {
            basicPoolStats.setNotSendedBack(basicPoolStats.getNotSendedBack() - 1);
        }
    }

    public void copyStats(final BasicPoolStats stats) {
        synchronized (basicPoolStats) {
            stats.setNotSendedBack(basicPoolStats.getNotSendedBack());
            stats.setWaiters(basicPoolStats.getWaiters());
        }
    }

}
