/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.listener.statistics;

import org.ow2.util.pool.impl.enhanced.api.IWaitControl;


/**
 *
 * @author Gael Lalire
 * @param <E> pool item type
 */
public class ExtendedPoolStatsFillerPoolListener<E> extends BasicPoolStatsFillerPoolListener<E> {

    private ThreadLocal<Long> callTime;

    private ExtendedPoolStats extendedPoolStats;

    public ExtendedPoolStatsFillerPoolListener() {
        callTime = new ThreadLocal<Long>();
        extendedPoolStats = new ExtendedPoolStats();
    }

    @Override
    public void getMethodCalled(final IWaitControl timeout) {
        callTime.set(System.currentTimeMillis());
        super.getMethodCalled(timeout);
    }

    protected void updateGetMethodDurationAverage() {
        long duration = System.currentTimeMillis() - callTime.get();
        callTime.remove();
        long oldCount = extendedPoolStats.getGetMethodCount();
        double getMethodDurationAverage = (extendedPoolStats.getGetMethodDurationAverage() * oldCount + duration)
                / (oldCount + 1);
        extendedPoolStats.setGetMethodDurationAverage(getMethodDurationAverage);
        extendedPoolStats.setGetMethodCount(oldCount + 1);
    }

    @Override
    public void getMethodFailed(final Exception poolException, final IWaitControl timeout) {
        updateGetMethodDurationAverage();
        super.getMethodFailed(poolException, timeout);
    }

    @Override
    public void getMethodReturned(final E result, final IWaitControl timeout) {
        updateGetMethodDurationAverage();
        super.getMethodReturned(result, timeout);
    }

}
