/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.thread;

import java.lang.ref.WeakReference;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.PoolException;
import org.ow2.util.pool.impl.enhanced.api.thread.IReusableThread;

/**
 * @author Gael Lalire
 */
public class AbstractReusableThread implements IReusableThread, Runnable {

    private static final Log LOG = LogFactory.getLog(AbstractReusableThread.class);

    private Runnable runnable;

    private WeakReference<IPool<IReusableThread>> reusableThreadPoolWeakReference;

    private Object mutex;

    public AbstractReusableThread() {
        mutex = new Object();
    }

    public void setUsed(final IPool<IReusableThread> reusableThreadPool, final Runnable runnable) {
        if (reusableThreadPool == null || runnable == null) {
            throw new IllegalArgumentException();
        }
        synchronized (mutex) {
            if (this.runnable != null) {
                throw new IllegalStateException();
            }
            this.reusableThreadPoolWeakReference = new WeakReference<IPool<IReusableThread>>(reusableThreadPool);
            this.runnable = runnable;
        }
    }

    public void run() {
        try {
            runnable.run();
        } catch (RuntimeException e) {
            LOG.error("Runnable exit with runtime exception", e);
        } catch (Error e) {
            LOG.error("Runnable exit with error, this thread is lost", e);
            throw e;
        }
        synchronized (mutex) {
            runnable = null;
            IPool<IReusableThread> reusableThreadPool = reusableThreadPoolWeakReference.get();
            if (reusableThreadPool != null) {
                try {
                    reusableThreadPool.put(this);
                } catch (PoolException e) {
                    LOG.error("Pool refuse the item", e);
                }
            }
            reusableThreadPoolWeakReference = null;
        }
    }

}
