/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.thread.workmanager;

import javax.resource.spi.work.Work;
import javax.resource.spi.work.WorkException;
import javax.resource.spi.work.WorkManager;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.thread.IReusableThread;
import org.ow2.util.pool.impl.enhanced.impl.thread.AbstractReusableThread;

/**
 *
 * @author Gael Lalire
 */
public class WorkManagerReusableThread extends AbstractReusableThread implements Work {

    private static final Log LOG = LogFactory.getLog(WorkManagerReusableThread.class);

    private WorkManager workManager;

    public WorkManagerReusableThread(final WorkManager workManager) {
        this.workManager = workManager;
    }

    @Override
    public void setUsed(final IPool<IReusableThread> reusableThreadPool, final Runnable runnable) {
        super.setUsed(reusableThreadPool, runnable);
        try {
            // we don't care is there is no available thread
            // this is better to return speedly
            workManager.scheduleWork(this);
        } catch (WorkException e) {
            LOG.error("Unable to schedule a work", e);
        }
    }

    public void release() {
    }

}
