/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.util;

import java.util.concurrent.Executor;
import java.util.concurrent.atomic.AtomicLong;

/**
 *
 * @author Gael Lalire
 */
public final class ExecutorProvider {

    private static AtomicLong instanceCount = new AtomicLong(0);

    private ExecutorProvider() {
    }

    public static final Executor SELF_THREAD_EXECUTOR = new Executor() {

        public void execute(final Runnable command) {
            command.run();
        }

    };

    public static final Executor NEW_THREAD_EXECUTOR = new Executor() {

        public void execute(final Runnable command) {
            new Thread(command, "NewThreadExecutor " + instanceCount.getAndIncrement()).start();
        }

    };

}
