/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.validator;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.NotABusyPoolItemException;
import org.ow2.util.pool.impl.enhanced.api.PoolException;
import org.ow2.util.pool.impl.enhanced.api.TimeoutPoolException;
import org.ow2.util.pool.impl.enhanced.api.validator.IPoolValidator;
import org.ow2.util.pool.impl.enhanced.impl.PoolError;

/**
 * @author Gael Lalire
 * @param <E> pool item type
 */
public class ValidatorPool<E> implements IPool<E> {

    private static final Log LOG = LogFactory.getLog(ValidatorPool.class);

    private IPool<E> pool;

    private IPoolValidator<E> poolValidator;

    private boolean putValidate;

    public ValidatorPool(final IPool<E> pool, final IPoolValidator<E> poolValidator, final boolean putValidate) {
        this.pool = pool;
        this.poolValidator = poolValidator;
        this.putValidate = putValidate;
    }

    protected boolean validateOneItem(final E poolItem) {
        try {
            return poolValidator.validate(poolItem);
        } catch (RuntimeException e) {
            LOG.error("Validation failed", e);
            return false;
        }
    }

    public E get(final IWaitControl timeout) throws PoolException, InterruptedException {
        E poolItem = pool.get(timeout);
        while (!validateOneItem(poolItem)) {
            try {
                pool.remove(poolItem);
            } catch (NotABusyPoolItemException e) {
                throw new PoolError("Delegated pool has a bad behavior");
            }
            if (!timeout.canContinueToWait()) {
                throw new TimeoutPoolException();
            }
            poolItem = pool.get(timeout);
        }
        return poolItem;
    }

    public void signalAllWaiters() {
        pool.signalAllWaiters();
    }

    public void put(final E poolItem) throws NotABusyPoolItemException {
        if (putValidate && !validateOneItem(poolItem)) {
            pool.remove(poolItem);
        } else {
            pool.put(poolItem);
        }
    }

    public void remove(final E poolItem) throws NotABusyPoolItemException {
        pool.remove(poolItem);
    }

}
