/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.validator.clue;

import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.NotABusyPoolItemException;
import org.ow2.util.pool.impl.enhanced.api.PoolException;
import org.ow2.util.pool.impl.enhanced.api.TimeoutPoolException;
import org.ow2.util.pool.impl.enhanced.api.clue.ICluePool;
import org.ow2.util.pool.impl.enhanced.api.validator.clue.ICluePoolValidator;
import org.ow2.util.pool.impl.enhanced.impl.PoolError;
import org.ow2.util.pool.impl.enhanced.impl.validator.ValidatorPool;

/**
 *
 * @author Gael Lalire
 * @param <E> pool item type
 * @param <C> clue type
 */
public class ValidatorCluePool<E, C> extends ValidatorPool<E> implements ICluePool<E, C> {

    private ICluePool<E, C> cluePool;

    private ICluePoolValidator<E, C> cluePoolValidator;

    public ValidatorCluePool(final ICluePool<E, C> cluePool, final ICluePoolValidator<E, C> cluePoolValidator,
            final boolean putValidate) {
        super(cluePool, cluePoolValidator, putValidate);
        this.cluePool = cluePool;
        this.cluePoolValidator = cluePoolValidator;
    }

    public E get(final C clue, final IWaitControl timeout) throws PoolException, InterruptedException {
        E poolItem = cluePool.get(clue, timeout);
        while (!cluePoolValidator.validate(poolItem, clue)) {
            try {
                cluePool.remove(poolItem);
            } catch (NotABusyPoolItemException e) {
                throw new PoolError("Delegated pool has a bad behavior");
            }
            if (!timeout.canContinueToWait()) {
                throw new TimeoutPoolException();
            }
            poolItem = cluePool.get(timeout);
        }
        return poolItem;
    }

}
