/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.internal.lock.impl;

import java.util.Date;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;

import org.ow2.util.pool.impl.enhanced.internal.lock.api.ISignalClearableCondition;

/**
 *
 * @author Gael Lalire
 */
public class SignalClearableConditionProxy implements ISignalClearableCondition {

    private Condition condition;

    public SignalClearableConditionProxy(final Condition condition) {
        if (condition == null) {
            throw new IllegalArgumentException("condition is null");
        }
        this.condition = condition;
    }

    /**
     * Use this to create a proxy which do nothing on clearAllSignal.
     * @param condition
     * @return
     */
    public static ISignalClearableCondition createProxy(final Condition condition) {
        if (condition instanceof ISignalClearableCondition) {
            return (ISignalClearableCondition) condition;
        }
        return new SignalClearableConditionProxy(condition);
    }

    /**
     * Do nothing.
     */
    public void clearAllSignal() {
    }

    public void await() throws InterruptedException {
        condition.await();
    }

    public boolean await(final long time, final TimeUnit unit) throws InterruptedException {
        return condition.await(time, unit);
    }

    public long awaitNanos(final long nanosTimeout) throws InterruptedException {
        return condition.awaitNanos(nanosTimeout);
    }

    public void awaitUninterruptibly() {
        condition.awaitUninterruptibly();
    }

    public boolean awaitUntil(final Date deadline) throws InterruptedException {
        return condition.awaitUntil(deadline);
    }

    public void signal() {
        condition.signal();
    }

    public void signalAll() {
        condition.signalAll();
    }

    public void await(final long timeout) throws InterruptedException {
        if (timeout < 0) {
            throw new IllegalArgumentException();
        }
        if (timeout == 0) {
            condition.await();
        } else {
            condition.await(timeout, TimeUnit.MILLISECONDS);
        }
    }

    public void signalAllUnclearable() {
        condition.signalAll();
    }

}
