/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.internal.resizer.api;


/**
 * A pool resizer can only manage one pool.
 * @author Gael Lalire
 * @param <S> statistics type
 */
public interface IResizer<S> {

    /**
     * Prepare an update (calculate next value of expectedSize).
     * When this method returns the calcul must be complete (no asynchronous mechanism can be used).
     *
     * @param stats statistics
     */
     void prepareUpdate(S stats);

    /**
     * This method grant that eventually at least one call will
     * be made to setExpectedSize, with stats calculated by latest call to
     * prepareUpdate.
     * When this method returns you cannot know if the setExpectedSize have been called (asynchronous mechanism
     * like watchDog or delegation to another thread can be used).
     */
     void update();

     /**
      * Resizer will call method on it.
      * @param resizable a resizable
      */
     void setResizable(final IResizable resizable);

}
