/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.internal.resizer.impl;

import org.ow2.util.pool.impl.enhanced.internal.resizer.api.IResizable;
import org.ow2.util.pool.impl.enhanced.internal.resizer.api.IResizer;

/**
 *
 * @author Gael Lalire
 * @param <S> statistics type
 */
public abstract class AbstractResizer<S> implements IResizer<S> {

    private Object mutex;

    private IResizable resizable;

    public AbstractResizer() {
        mutex = new Object();
        this.resizable = null;
    }

    protected void updateWith(final int expectedSize) {
        IResizable resizable = null;
        synchronized (mutex) {
            if (this.resizable != null) {
                resizable = this.resizable;
            }
        }
        if (resizable != null) {
            resizable.setExpectedSize(expectedSize);
        }
    }

    public void setResizable(final IResizable resizablePool) {
        synchronized (mutex) {
            this.resizable = resizablePool;
        }
    }

}
