/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.internal.resizer.impl;

import org.ow2.util.pool.impl.enhanced.internal.resizer.api.IResizable;
import org.ow2.util.pool.impl.enhanced.internal.resizer.api.IResizer;
import org.ow2.util.pool.impl.enhanced.internal.thread.impl.ManagementThread;

/**
 *
 * @author Gael Lalire
 * @param <S> statistics type
 */
public class AsynchronousResizer<S> implements IResizer<S>, Runnable {

    private ManagementThread[] managementThreads;

    private IResizer<S> resizer;

    private boolean willRun;

    public AsynchronousResizer(final IResizer<S> poolResizer) {
        this(poolResizer, 1);
    }

    public AsynchronousResizer(final IResizer<S> poolResizer, final int threadCount) {
        this.resizer = poolResizer;
        managementThreads = new ManagementThread[threadCount];
        for (int i = 0; i < threadCount; i++) {
            managementThreads[i] = new ManagementThread(this);
        }
    }

    public void prepareUpdate(final S state) {
        resizer.prepareUpdate(state);
    }

    public void update() {
        synchronized (managementThreads) {
            willRun = true;
        }
        for (int i = 0; i < managementThreads.length; i++) {
            managementThreads[i].start();
        }
    }

    public void run() {
        boolean doUpdate;
        synchronized (managementThreads) {
            doUpdate = willRun;
        }
        if (doUpdate) {
            resizer.update();
        }
    }

    public void setResizable(final IResizable resizable) {
        synchronized (managementThreads) {
            willRun = false;
        }
        resizer.setResizable(resizable);
    }

}
