/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.internal.resizer.impl.shared;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.pool.impl.enhanced.internal.resizer.api.IResizer;
import org.ow2.util.pool.impl.enhanced.internal.resizer.api.shared.ISharedManager;
import org.ow2.util.pool.impl.enhanced.internal.thread.impl.ManagementThread;

/**
 * @author Gael Lalire
 */
public class SharedResizerOneThreadManager implements ISharedManager, Runnable {

    private List<IResizer<?>> resizerList;

    private ManagementThread managementThread;

    private boolean running;

    public SharedResizerOneThreadManager() {
        resizerList = new ArrayList<IResizer<?>>();
        managementThread = new ManagementThread(this);
        running = false;
    }

    public void update(final IResizer<?> resizer) {
        synchronized (resizerList) {
            if (!running) {
                // if management thread is not started then start it
                managementThread.start();
            }
            resizerList.add(resizer);
        }
    }

    public void run() {
        synchronized (resizerList) {
            running = true;
        }
        while (true) {
            IResizer<?> resizer;
            synchronized (resizerList) {
                if (resizerList.isEmpty()) {
                    running = false;
                    return;
                }
                resizer = resizerList.remove(0);
            }
            resizer.update();
        }
    }

    /**
     * @return the resizerList
     */
    protected List<IResizer<?>> getResizerList() {
        return resizerList;
    }

    /**
     * @return the managementThread
     */
    protected ManagementThread getManagementThread() {
        return managementThread;
    }


    /**
     * @return the running
     */
    protected boolean isRunning() {
        return running;
    }

}
