/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.internal.resizer.impl.shared;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.PoolException;
import org.ow2.util.pool.impl.enhanced.api.thread.IReusableThread;
import org.ow2.util.pool.impl.enhanced.impl.waitcontrol.NoWaitControl;
import org.ow2.util.pool.impl.enhanced.internal.resizer.api.IResizer;

/**
 *
 * @author Gael Lalire
 */
public class SharedResizerPoolThreadManager extends SharedResizerOneThreadManager {

    private static final Log LOG = LogFactory.getLog(SharedResizerPoolThreadManager.class);

    private IPool<IReusableThread> reusableThreadPool;

    public SharedResizerPoolThreadManager(final IPool<IReusableThread> reusableThreadPool) {
        this.reusableThreadPool = reusableThreadPool;
    }

    @Override
    public void update(final IResizer<?> resizer) {
        // the thread manager will update size
        super.update(resizer);
        // but if thread manager is busy with another resizer
        // instead of waiting for it we ask a thread from the
        // thread pool to do the same stuff
        try {
            // after run on this is done the IReusableThread send back itself in the pool
            reusableThreadPool.get(NoWaitControl.INSTANCE).setUsed(reusableThreadPool, this);
        } catch (PoolException e) {
            LOG.debug("Unable to get a pool item, manager thread will be used", e);
        } catch (InterruptedException e) {
            LOG.debug("Unable to get a pool item, manager thread will be used", e);
        }
    }

}
