/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.internal.store;

import java.io.File;

import org.ow2.util.stream.api.IStreamAccessor;
import org.ow2.util.stream.impl.file.SystemFileAccessor;


/**
 *
 * @author Gael Lalire
 */
public class FilesInFolderStoreMechanism implements StoreMechanism {

    private long nextId;

    private File directory;

    private String prefix;

    private String suffix;

    private Object mutex;

    public FilesInFolderStoreMechanism(final File directory, final String prefix, final String suffix) {
        if (prefix == null || suffix == null) {
            throw new IllegalArgumentException();
        }
        if (directory == null || !directory.isDirectory()) {
            throw new IllegalArgumentException();
        }
        this.directory = directory;
        this.prefix = prefix;
        this.suffix = suffix;
        mutex = new Object();
    }

    public IStreamAccessor getNewStreamAccessor() {
        long currentId;
        synchronized (mutex) {
            currentId = nextId;
            nextId++;
        }
        return new SystemFileAccessor(new File(directory, prefix + currentId + suffix));
    }

}
