/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl.enhanced.impl.keepbusy;

import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;

/**
 * @author Gael Lalire
 * @param <E> reference type
 */
public final class IdentityWeakReference<E> extends WeakReference<E> implements IIdentityReference<E> {

    private int hashCode;

    public IdentityWeakReference(final E reference, final ReferenceQueue<E> referenceQueue) {
        super(reference, referenceQueue);
        hashCode = System.identityHashCode(reference);
    }

    @Override
    public boolean equals(final Object object) {
        if (this == object) {
            return true;
        }
        if (!(object instanceof IIdentityReference<?>)) {
            return false;
        }
        Object reference = ((IIdentityReference<?>) object).getReference();
        Object localObject = get();
        if (localObject == null) {
            return false;
        }
        if (localObject != reference) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        return hashCode;
    }

    public E getReference() {
        return get();
    }
}
